require("dotenv").config();
const cors = require("cors");
const express = require("express");
const SSLCommerzPayment = require("sslcommerz-lts");
const {v4: uuidv4} = require('uuid');
const res = require("express/lib/response");
const mysql = require('mysql2');

const pool = mysql.createPool({
    host:'localhost:3306',
    user: 'diancavm_root',
    password: '0+UgT1eDlTxO',
    database: 'diancavm_jurag44',
    waitForConnections: true,
    connectionLimit: 10,  // Maximum number of connections in the pool
    queueLimit: 0        // Unlimited queue length for requests
});
// SSLCommerz credentials
const store_id = "mh66fee92bd5d2f";
const store_passwd = "mh66fee92bd5d2f@ssl";
const is_live = "";

const createTable = async () => {

    const createTableQuery = `
     CREATE TABLE IF NOT EXISTS payments (
      tran_id VARCHAR(36) PRIMARY KEY,
      mobileNumber VARCHAR(15),
      name VARCHAR(100),
      email VARCHAR(100),
      registration VARCHAR(50),
      amount DECIMAL(10, 2),
      faculty VARCHAR(100),
      department VARCHAR(100),
      tShirtSize VARCHAR(5),
      Spouse BOOLEAN,
      currency VARCHAR(10),
      payment_status VARCHAR(100)
    )
  `;

    try {
        await pool.query(createTableQuery);
        return 200;
    } catch (e) {
        console.log(e);
        return 500;

    }
};

async function addPaymentRecord({
                                    tran_id,
                                    mobileNumber,
                                    name,
                                    email,
                                    registration,
                                    amount,
                                    faculty,
                                    department,
                                    tShirtSize,
                                    Spouse,
                                    currency,
                                    payment_status,
                                }) {
    // Get a connection from the pool
    const insertQuery = `
    INSERT INTO payments (tran_id, mobileNumber, name, email, registration, amount, faculty, department, tShirtSize, Spouse, currency, payment_status)
    VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
  `;

    try {
        const results = await pool.query(insertQuery, [
            tran_id,
            mobileNumber,
            name,
            email,
            registration,
            amount,
            faculty,
            department,
            tShirtSize,
            Spouse,
            currency,
            payment_status,
        ]);
        return 200;
    } catch (error) {
        console.error('Error inserting row:', error);
        return 500;
    }
}

async function updatedPaymentStatus({
                                        tran_id,
                                        payment_status,
                                    }) {
    // Get a connection from the pool
    const updateQuery = `
    UPDATE payments
    SET payment_status = ?
    WHERE tran_id = ?
  `;

    console.log("tran_id", tran_id);

    try {
        const [result] = await pool.query(updateQuery, [payment_status, tran_id]);

        if (result.affectedRows > 0) {
            console.log("Payment Status Update Successful")
            return 200;
        } else {
            return 404;
        }
    } catch (error) {
        console.error('Error updating Update Status: ', error);
        return 500;
    }
}


const app = express();
const port = 3001;

app.use(cors());
app.use(express.json());

async function run() {
    try {
        const tableCreated = await createTable();
        console.log("tableCreated ", tableCreated);

        app.get("/health", (_req, res) => {
            res.status(200).send({
                status: "success",
                message: "Your server is up now."
            })
        });

        app.post("/payment", async (req, res) => {
            const {
                mobileNumber = '',
                name = '',
                email = '',
                registration = '',
                amount = 0,
                faculty = '',
                department = '',
                tShirtSize = '',
                Spouse = '',
                currency = 'BDT',
            } = req.body;

            if (!Number.isFinite(amount)) {
                return res.status(400).send({
                    status: "error",
                    message: "Amount should be a number.",
                })
            }
            if (amount <= 0) {
                return res.status(400).send({
                    status: "error",
                    message: "Amount should be more than 0",
                })
            }

            const tran_id = uuidv4();
            const payment_status = "PENDING";
            const data = {
                total_amount: amount,
                currency,
                tran_id, // use unique tran_id for each api call
                success_url: `${process.env.SDURL}/payment/success/${tran_id}`,
                fail_url: `${process.env.SDURL}/payment/fail/${tran_id}`,
                cancel_url: `${process.env.SDURL}/payment/cancel/${tran_id}`,
                ipn_url: `${process.env.SDURL}/payment/ipn/${tran_id}`,
                shipping_method: 'Courier',
                product_name: 'JURAG44',
                product_category: 'RAG',
                product_profile: 'RAG',
                cus_name: name,
                cus_email: email,
                cus_add1: 'Dhaka',
                cus_add2: 'Dhaka',
                cus_city: 'Dhaka',
                cus_state: 'Dhaka',
                cus_postcode: '1000',
                cus_country: 'Bangladesh',
                cus_phone: mobileNumber,
                cus_fax: 'N/A',
                ship_name: name,
                ship_add1: 'Dhaka',
                ship_add2: 'Dhaka',
                ship_city: 'Dhaka',
                ship_state: 'Dhaka',
                ship_postcode: 1342,
                ship_country: 'Bangladesh',
            };


            const sslcz = new SSLCommerzPayment(store_id, store_passwd, is_live === "true");

            try {
                const apiResponse = await sslcz.init(data);
                console.log(apiResponse.GatewayPageURL, 'api response');
                const createdRecord = await addPaymentRecord({
                    tran_id: tran_id.trim(),
                    mobileNumber: mobileNumber.trim(),
                    name: name.trim(),
                    email: email.trim(),
                    registration: registration.trim(),
                    amount,
                    faculty: faculty.trim(),
                    department: department.trim(),
                    tShirtSize: tShirtSize.trim(),
                    Spouse: Spouse === true || Spouse === "true",
                    currency,
                    payment_status,
                });
                console.log(createdRecord);
                if (createdRecord === 200) {
                    return res.send({status: "success", url: apiResponse.GatewayPageURL});
                }

                return res.status(500).send({status: "error", message: "Internal server error"});

            } catch (error) {
                console.log(error);

                return res.status(500).send({status: "error", message: "Internal server error"});
            }
        });

        app.post("/payment/success/:tran_id", async (req, res) => {
            const tran_id = req.params.tran_id;
            const payment_status = "WAITING_FOR_IPN";
            try {
                const updatedResult = await updatedPaymentStatus({tran_id, payment_status})
                if (updatedResult === 200) {
                    return res.redirect('https://ju44.xyz/#/success');
                } else if (updatedResult === 404) {
                    return res.status(404).send({
                        status: "error",
                        message: "Transaction not found.",
                    });

                }


                return res.status(500).send({status: "error", message: "Internal server error"});

            } catch (e) {
                console.log(e);

                return res.status(500).send({status: "error", message: "Internal server error"});

            }

        });

        app.post("/payment/fail/:tran_id", async (req, res) => {
            const tran_id = req.params.tran_id;
            const payment_status = "FAILED";
            try {
                const updatedResult = await updatedPaymentStatus({tran_id, payment_status})
                if (updatedResult === 200) {
                    return res.redirect('https://ju44.xyz/#/fail');
                } else if (updatedResult === 404) {
                    return res.status(404).send({
                        status: "error",
                        message: "Transaction not found.",
                    });

                }


                return res.status(500).send({status: "error", message: "Internal server error"});

            } catch (e) {
                console.log(e);

                return res.status(500).send({status: "error", message: "Internal server error"});

            }

        });

        app.post("/payment/ipn/:tran_id", async (req, res) => {
            const tran_id = req.params.tran_id;
            const payment_status = "IPN_RECEIVED";
            try {
                const updatedResult = await updatedPaymentStatus({tran_id, payment_status})
                if (updatedResult === 200) {
                    return res.status(200).send(200);
                } else if (updatedResult === 404) {
                    return res.status(404).send({
                        status: "error",
                        message: "Transaction not found.",
                    });

                }


                return res.status(500).send({status: "error", message: "Internal server error"});

            } catch (e) {
                console.log(e);

                return res.status(500).send({status: "error", message: "Internal server error"});

            }
        });

        app.post("/payment/cancel/:tran_id", async (req, res) => {
            const tran_id = req.params.tran_id;
            const payment_status = "CANCELLED";
            try {
                const updatedResult = await updatedPaymentStatus({tran_id, payment_status})
                if (updatedResult === 200) {
                    return res.redirect('https://ju44.xyz/#/cancel');
                } else if (updatedResult === 404) {
                    return res.status(404).send({
                        status: "error",
                        message: "Transaction not found.",
                    });

                }


                return res.status(500).send({status: "error", message: "Internal server error"});

            } catch (e) {
                console.log(e);

                return res.status(500).send({status: "error", message: "Internal server error"});

            }
        });
        app.get('/get-payments', async (req, res) => {
            try {
                // Query to select all records from the payments table
                const [rows] = await pool.query('SELECT * FROM payments');

                // Send the result as a JSON response
                res.json(rows);
            } catch (error) {
                console.error('Error fetching payments:', error);
                res.status(500).json({ error: 'An error occurred while fetching payments.' });
            }
        });

        // app.get("/order-details/:userId", async (req, res) => {
        //     const userId = req.params.userId;
        //     const findOrder = await orderCollection.find({"extraInformation.customersId": userId}).toArray();
        //     res.json({data: "success", order: findOrder});
        // });
    } catch (error) {
        console.error("Error connecting to database:", error);
    }
}

run();

app.listen(port, () => {
    console.log(`Listening on port ${port}`);
});